#include <stdio.h>
#include <stdlib.h>
#include <dlfcn.h>
#include <unistd.h>
#include <asm/fcntl.h>
#include <sys/stat.h>


static int soundfd = -1;
static int logfile = -1;


void *
loadfunc (const char *name)
  {
    char *error;
    void *funcptr;
    static void *handle=NULL;

    if (!handle)
      {
        handle = dlopen ("libc.so.6", RTLD_LAZY);
        if (!handle)
          {
            fputs (dlerror (), stderr);
            exit(1);
          }
      }

    funcptr = dlsym(handle, name);
    if ((error = dlerror ()) != NULL)
      {
        fprintf (stderr, "%s\n", error);
        exit(1);
      }

    return funcptr;
  }


int
open (const char *pathname, int flags)
  {
    static int (*openfunc) (const char *pathname, int flags) = NULL;

    int ret_val;

    if (!openfunc)
      {
        /* this runs once to get the function from the libc */
        openfunc = loadfunc ("open");
      }
    
    ret_val = (*openfunc) (pathname, flags);

    if (!strcmp ("/dev/dsp", pathname))
      {
        fprintf (stderr, "opened Sounddevice, filedeskriptor %d\n", ret_val);
        soundfd = ret_val;
        logfile = (*openfunc) ("/tmp/audiograb.raw", O_WRONLY|O_CREAT|O_APPEND);
        fchmod (logfile, S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH);
      }

    return ret_val;
  }



int
close (int fd)
  {
    static int (*closefunc) (int fd) = NULL;

    int ret_val;

    if (!closefunc)
      {
        /* this runs once to get the function from the libc */
        closefunc = loadfunc ("close");
      }
    
    /* Dont close if it is our audiofile */
    if (soundfd >= 0 && fd == logfile)
      return 0;

    if (logfile >= 0 && soundfd == fd)
      {
        fprintf (stderr, "closed Sounddevice\n");
        (*closefunc) (logfile);
      }

    ret_val = (*closefunc) (fd);

    return ret_val;
  }



ssize_t
write (int fd, const void *buf, size_t count)
  {
    static ssize_t (*writefunc) (int fd, const void *buf, size_t count) = NULL;

    ssize_t ret_val;

    if (!writefunc)
      {
        /* this runs once to get the function from the libc */
        writefunc = loadfunc ("write");
      }
    
    ret_val = (*writefunc) (fd, buf, count);

    /* Hoffentlich geht das gut... */
    if (ret_val >= 0 && fd == soundfd && logfile >= 0)
      (*writefunc) (logfile, buf, ret_val);

    return ret_val;
  }


